from __future__ import absolute_import, print_function, unicode_literals

from . import (
    django1_6,
    django1_7,
    django1_8,
    django1_9,
    django1_10,
    django1_11,
    django2_0,
    django2_1,
    django2_2,
    django3_0,
    django3_1,
)
