from __future__ import absolute_import, print_function, unicode_literals


# NOTE : No fix provided for "Admin actions are no longer collected from base ModelAdmin classes" for now, since it seems no one really relied on this behaviour?

